package mastermind.engine;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import mastermind.logic.RandomGuess;
import mastermind.logic.Serializer;
import mastermind.logic.SettingsHandler;
import mastermind.model.Guess;
import mastermind.model.Result;
import mastermind.model.Settings;

/**
 * Mastermind Engine: Enthält die Logik für das Spiel.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-04-29.v01   FeRo    init version
 * 2012-04-30.v01   FeRo    validateSettings(), Exception Handling
 * 2012-05-03.v01   FeRo    Settings ausgelagert
 * 2012-05-05.v01   FeRo    SettingsHandler
 * 2012-05-18.v01   FeRo    History, Matrix
 * 2012-05-19.v01   FeRo    Load / Save
 * -----------------------------------------------------------------------------
 * ToDo:
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-19.v01
 */
public class MastermindEngine implements IMastermindEngine, Serializable
{
    // Mastermind Options

    private Settings settings; // Spiel Optionen
    private Guess hiddenPattern; // das zu erratende Muster
    private ArrayList<Guess> historyGuess; // ArrayList mit Guesses
    private ArrayList<Result> historyResult; // ArrayList mit Resultaten

    /**
     * Mastermind Engine Konstruktor
     */
    public MastermindEngine()
    {
        this.settings = SettingsHandler.getInstance().getSettings();
        this.hiddenPattern = RandomGuess.genRandomGuess();
        historyGuess = new ArrayList<Guess>();
        historyResult = new ArrayList<Result>();
    }

    /**
     * Gibt zurück ob gewonnen oder nicht. Vergleicht Guess == hiddenPattern
     * @param guess Kombination die geraten wurde
     * @return ob gewonnen oder nicht
     */
    @Override
    public boolean hasWon(Guess guess)
    {
        return this.hiddenPattern.equals(guess);
    }

    /**
     * Berechnet die Anzahl Stifte mit richtiger Farbe und richtiger Position (schwarz) sowie nur richtigen Farbe (weiss) sowie ob Gewonnen oder nicht.
     * @param guess Kombination die geraten wurde
     * @return Result mit den Anzahl Schwarzen / Weissen stifte
     */
    @Override
    public Result getResult(Guess guess)
    {
        // Guess in History speichern
        historyGuess.add(guess);
        // Resultat berechnen
        Result guessResult = new Result();
        if (hasWon(guess)) {
            // gewonnen => alles richtig
            guessResult.setAnzPegColorPosRight(settings.getNumberOfPegs());
        } else {
            // Berechnung Anzahl Schwarz / Weiss
            int[] codePegs = hiddenPattern.getPegs();
            int[] guessPegs = guess.getPegs();
            boolean[] codePegUsed = new boolean[settings.getNumberOfPegs()];
            boolean[] guessPegUsed = new boolean[settings.getNumberOfPegs()];
            // Überprüfe ob Farbe & Position richtig ==> Schwarze Stifte
            for (int i = 0; i < settings.getNumberOfPegs(); i ++) {
                if (( ! codePegUsed[i]) && (guessPegs[i] == codePegs[i])) {
                    // farbe und position stimmt überein
                    guessResult.incAnzPegColorPosRight();
                    codePegUsed[i] = true;
                    guessPegUsed[i] = true;
                }
            }
            // Überprüfe ob Farbe richtig ==> Weisse Stifte
            for (int i = 0; i < settings.getNumberOfPegs(); i ++) {
                for (int j = 0; j < settings.getNumberOfPegs(); j ++) {
                    if (( ! guessPegUsed[i] &&  ! codePegUsed[j]) && (guessPegs[i] == codePegs[j])) {
                        // farbe stimmt überein
                        guessResult.incAnzPegColorRight();
                        guessPegUsed[i] = true;
                        codePegUsed[j] = true;
                    }
                }
            }
        }
        // Result in History speichern
        historyResult.add(guessResult);
        return (guessResult);
    }

    /**
     * Gibt das zu erratende Muster (Guess) zurück
     * @return the hiddenPattern
     */
    public Guess getHiddenPattern()
    {
        return hiddenPattern;
    }

    /**
     * Setzt das zu erratende Muster (Guess)
     * @param hiddenPattern the hiddenPattern to set
     * @param showCode true: shows Hidden Code
     */
    @Override
    public void setHiddenPattern(Guess hiddenPattern, boolean showCode)
    {
        this.hiddenPattern = hiddenPattern;
        if (showCode) {
            System.out.println("Code: " + hiddenPattern);
        }
    }

    /**
     * Hilfsmethode: Convert String in ein Int-Array
     * @param guess
     * @return Int-Array
     */
    public static int[] convertStringToIntArr(String guess)
    {
        int[] res = new int[guess.length()];
        if (guess != null) {
            for (int i = 0; i <
                guess.length(); i ++) {
                try {
                    res[i] = Integer.parseInt(String.valueOf(guess.charAt(i)));
                } catch (Exception e) {
                    // abort...
                    return res;
                }
            }
            return res;
        }
        return res;
    }

    /**
     * Gibt die Anzahl bereits geratenen Rateversuche zurück
     * @return Anzahl Rateversuche
     */
    public int getNumberOfGuesses()
    {
        return historyGuess.size();
    }

    /**
     * Gibt einen Guess zurück aus der History
     * @param guessNumber Position von Guess
     * @return Guess Objekt
     */
    public Guess getHistoryGuess(int guessNumber)
    {
        if ((guessNumber >= 0) && (guessNumber <= historyGuess.size())) {
            return historyGuess.get(guessNumber);
        } else {
            return null;
        }
    }

    /**
     * Gibt das Resultat eines Guesses aus der History zurück
     * @param resultForGuessNumber Position des Guess
     * @return Result Objekt
     */
    public Result getHistoryResult(int resultForGuessNumber)
    {
        if ((resultForGuessNumber >= 0) && (resultForGuessNumber <= historyResult.size())) {
            return historyResult.get(resultForGuessNumber);
        } else {
            return null;
        }
    }

    /**
     * Mastermind Engine reset: alle History einträge löschen, neuer Code erzeugen, Settings neu laden
     */
    public void reset()
    {
        System.out.println("\n");
        System.out.println("Mastermind Engine: Reset...");
        this.settings = SettingsHandler.getInstance().getSettings();
        System.out.println("Secret Code wird erzeugt...");
        this.hiddenPattern = RandomGuess.genRandomGuess();
        System.out.println("Code: " + hiddenPattern);
        historyGuess.clear();
        historyResult.clear();
    }

    /**
     * Speichert die aktuelle Engine in eine Datei.
     * @param file Datei in welcher die Engine gespeichert werden soll.
     */
    public void saveEngine(File file)
    {
        System.out.println("Save Engine...");
        Serializer.Serialize(file, this);
    }

    /**
     * Lädt die Engine aus einer Datei
     * @param file Datei worin die Engine gespeichert ist.
     */
    public void loadEngine(File file)
    {
        System.out.println("Load Engine...");
        if (file.exists()) {
            try {
                MastermindEngine loadedEngine = (MastermindEngine) Serializer.DeSerialize(file);
                this.settings = loadedEngine.settings;
                this.hiddenPattern = loadedEngine.hiddenPattern;
                this.historyGuess = loadedEngine.historyGuess;
                this.historyResult = loadedEngine.historyResult;

                Settings mySettings = SettingsHandler.getInstance().getSettings();
                mySettings.setDesign(loadedEngine.settings.getDesign());
                mySettings.setNumberOfColors(loadedEngine.settings.getNumberOfColors());
                mySettings.setAllowDuplicateColor(loadedEngine.settings.getAllowDuplicateColor());
                mySettings.setNumberOfPegs(loadedEngine.settings.getNumberOfPegs());
                mySettings.setNumberOfRows(loadedEngine.settings.getNumberOfRows());

            } catch (Exception e) {
                // Error loading Enginge
                System.out.println("Error Loading Enginge!");
            }
            System.out.println("Code: " + this.hiddenPattern);
        } else {
            System.out.println("File not found: " + file);
        }
    }
}
