package mastermind.model;

import java.io.Serializable;
import java.util.Arrays;

/**
 * Klasse für eine Zeile, für einen Rate-Versuch / Hidden-Pattern.
 * Wird für ein einfacheres Verwalten von einem Rate-Versuch verwendet.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-04-29.v01   FeRo    init version
 * 2012-04-30.v01   FeRo    remove numberOfPegs - we don't need them...
 * 2012-05-04.v01   FeRo    add validateGuess()
 * 2012-05-05.v01   FeRo    GuessHanlder -> outsource validateGuess()
 * 2012-05-19.v01   FeRo    Serializable
 * 2012-05-28.v01   FeRo    compareTo für alternative Solver
 * 2012-05-30.v01   FeRo    Code CleanUp (alt. Solver removed)
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-30.v01
 */
public class Guess implements Serializable
{

    private int[] pegs;
    private int score;

    /**
     * Default Konstruktor. Anzahl Steckplätze: 1
     */
    public Guess()
    {
        this(new int[1]);
    }

    /**
     * Konstruktor, inkl Farben setzen
     * @param pegs Array mit den entsprechenden Farben pro Steckplatz
     */
    public Guess(int[] pegs)
    {
        this.pegs = pegs;
        int tmpScore = 0;
        int factor = 1;
        for (int i = pegs.length - 1; i >= 0; i --) {
            tmpScore += pegs[i] * factor;
            factor *= 10;
        }
        this.score = tmpScore;
    }

    /**
     * Konstruktor, inkl Farben und Score
     * @param pegs Array mit den entsprechenden Farben pro Steckplatz
     * @param score Score für diesen Guess
     */
    public Guess(int[] pegs, int score)
    {
        this.score = score;
        this.pegs = pegs;
    }

    /**
     * Setzt die einzelnen Farben der Steckplätze
     * @param pegs Array mit den zusetzenden Farben
     */
    public void setPegs(int[] pegs)
    {
        this.pegs = pegs;
    }

    /**
     * Gibt die einzelnen Farben zurück
     * @return Array mit den gesetzten Farben
     */
    public int[] getPegs()
    {
        return this.pegs;
    }

    /**
     * Setzt die Farbe von einem einzelnen Steckplatz
     * @param pos Position [0..n]
     * @param color Farbe an dieser Position
     */
    public void setPeg(int pos, int color)
    {
        if ((pos >= 0) && (pos < this.pegs.length)) {
            this.pegs[pos] = color;
        }
    }

    /**
     * Gibt die Aktuelle Farbe an der gewünschen Position zurück
     * @param pos gewünschte Position
     * @return int Farbe an dieser Position
     */
    public int getPeg(int pos)
    {
        return this.pegs[pos];
    }

    /**
     * Gibt die Score für diesen Guess zurück
     * @return Score von diesem Guess
     */
    public int getScore()
    {
        return score;
    }

    /**
     * Gibt eine Zeile als String zurück
     * @return guess als String
     */
    @Override
    public String toString()
    {
        return (Arrays.toString(pegs));
    }

    /**
     * Vergleich zwei Guess miteinander. Gibt True zurück wenn sie gleich sind.
     * @param otherGuess zu vergleichender Guess
     * @return gleicheit der zwei Guess
     */
    @Override
    public boolean equals(Object otherGuess)
    {
        if (this == otherGuess) {
            return true;
        }
        if (otherGuess == null) {
            return false;
        }
        if (otherGuess.getClass() != this.getClass()) {
            return false;
        }
        if ( ! Arrays.equals(this.pegs, ((Guess) otherGuess).pegs)) {
            return false;
        }
        return true;
    }

    /**
     * Vergleicht zwei Guess miteinander und gibt die Anzahl Stifte mit richtiger Farbe
     * und richtiger Position (schwarz) sowie nur richtigen Farbe (weiss)zurück.
     * @param otherGuess Kombination mit der verglichen werden soll
     * @return Result mit den Anzahl Schwarzen / Weissen Stifte
     */
    public Result compareGuess(Guess otherGuess)
    {
        Result guessResult = new Result();
        boolean won = true;
        int[] otherPegs = otherGuess.getPegs();
        boolean[] pegUsed = new boolean[pegs.length];

        // schwarz
        for (int i = 0; i < pegs.length; i ++) {
            if (pegs[i] == otherPegs[i]) {
                guessResult.incAnzPegColorPosRight();
                pegUsed[i] = true;
            } else {
                won = false;
            }
        }

        if ( ! won) {
            // weiss
            for (int i = 0; i < pegs.length; i ++) {
                if (pegs[i] != otherPegs[i]) {
                    for (int j = 0; j < pegs.length; j ++) {
                        if ((i != j) && ( ! pegUsed[j])) {
                            if (pegs[i] == otherPegs[j]) {
                                guessResult.incAnzPegColorRight();
                                pegUsed[j] = true;
                                break;
                            }
                        }
                    }
                }
            }
        }

        return (guessResult);
    }

    /**
     * Berechent den HashCode
     * @return HashCode des aktuellen Guess
     */
    @Override
    public int hashCode()
    {
        return Arrays.hashCode(this.pegs);
    }
}